from functools import wraps
import json

from timetappy.auth import TimeTapAuthentication
import requests


def return_response(fn):
    @wraps(fn)
    def wrapped(*args, **kwargs):
        response = fn(*args, **kwargs)
        if not response.status_code:
            return response.status_code
        return json.loads(response.text)
    return wrapped


def api_get_request(self, url, params: dict = {}):
    api_url = self.base_url + url
    response = requests.get(url=api_url, auth=TimeTapAuthentication(self.key, self.secret), params=params)
    return response


def api_post_request(self, url, params: dict = {}):
    api_url = self.base_url + url
    response = requests.post(url=api_url, auth=TimeTapAuthentication(self.key, self.secret), params=params)
    return response


def api_delete_request(self, url, params: dict = {}):
    api_url = self.base_url + url
    response = requests.delete(url=api_url, auth=TimeTapAuthentication(self.key, self.secret), params=params)
    return response


def api_patch_request(self, url, params: dict = {}):
    api_url = self.base_url + url
    response = requests.patch(url=api_url, auth=TimeTapAuthentication(self.key, self.secret), params=params)
    return response


def api_put_request(self, url, params: dict = {}):
    api_url = self.base_url + url
    response = requests.put(url=api_url, auth=TimeTapAuthentication(self.key, self.secret), params=params)
    return response



import requests

import datetime
import hashlib
import json
import logging
import inspect


class TimeTapAuthentication(requests.auth.AuthBase):

    logging.basicConfig(filename='auth.log', level=logging.DEBUG, format='%(asctime)s %(message)s')

    def __init__(self, APIKey: str, PrivateKey: str):
        self.APIKey = APIKey
        self.PrivateKey = PrivateKey
        self.timestamp = datetime.datetime.now().timestamp()

    def get_authorization_header(self):
        signature = hashlib.md5((self.APIKey + self.PrivateKey).encode('utf-8')).hexdigest()
        token = requests.get(url=f'https://api.timetap.com/live/sessionToken?apiKey={self.APIKey}&timestamp={self.timestamp}&signature={signature}')
        token = json.loads(token.text)
        return 'Bearer ' + token['sessionToken']

    def __call__(self, request):
        logging.info(f'Starting {inspect.currentframe().f_code.co_name}')
        request.headers['Authorization'] = self.get_authorization_header()
        logging.info(f'Request Authorization Header: {request.headers["Authorization"]}')
        logging.debug(f'URL: {request.url}')
        logging.debug(f'Request Headers: {request.headers.items()}')
        logging.info(f'Finishing {inspect.currentframe().f_code.co_name}')
        return request



from .utils import return_response, api_get_request


class AppointmentsMixin():
    TIMETAP_API_APPOINTMENTS = '/appointments/'
    TIMETAP_API_APPOINTMENTS_REPORT = '/appointments/report'

    api_get_request = api_get_request

    @return_response
    def get_appointments_report(self, statusList: list = ['OPEN'], pageSize: int = 1, pageNumber: int = 1):
        params = {
            'statusList': statusList,
            'pageSize': pageSize,
            'pageNumber': pageNumber
        }
        return self.api_get_request(f'{self.TIMETAP_API_APPOINTMENTS_REPORT}', params=params)

    @return_response
    def get_locations(self):
        return self.api_get_request(f'{self.TIMETAP_API_LOCATIONS}')

    @return_response
    def get_staff(self):
        return self.api_get_request(f'{self.TIMETAP_API_STAFF}')



from .utils import return_response, api_get_request


class StaffMixin():
    TIMETAP_API_STAFF = '/staff'

    api_get_request = api_get_request

    @return_response
    def get_staff(self):
        return self.api_get_request(f'{self.TIMETAP_API_STAFF}')

    @return_response
    def get_staff_by_professionalId(self, professionalId: int):
        if not professionalId:
            raise ValueError('professionalId has not been set')
        return self.api_get_request(f'{self.TIMETAP_API_STAFF}/{professionalId}')

    @return_response
    def get_service_staff(self, professionalId: int):
        if not professionalId:
            raise ValueError('professionalId has not been set')
        return self.api_get_request(f'{self.TIMETAP_API_STAFF}/{professionalId}/serviceStaff')



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="timetappy",
    version="0.0.1a",
    author="Abdullah 'AJ' Khan",
    author_email="aj@ajkhan.me",
    description="Unofficial API client wrapper around the TimeTap API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ak9999/timetappy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests>=2.23.0'
    ],
    python_requires='>=3.6',
)



from .utils import return_response, api_get_request


class ClientsMixin():
    TIMETAP_API_CLIENTS = '/clients'

    api_get_request = api_get_request

    @return_response
    def get_clients(self, clientId: int = None, order_field: str = None, order_mode: str = None,
                    pageNumber: int = 1, pageSize: int = 1):
        params = {
            'clientId': clientId, 'order_field': order_field,
            'order_mode': order_mode, 'pageNumber': pageNumber, 'pageSize': pageSize
        }
        return self.api_get_request(f'{self.TIMETAP_API_CLIENTS}', params=params)

    @return_response
    def get_clients_by_id(self, clientId: int = None):
        return self.api_get_request(f'{self.TIMETAP_API_CLIENTS}/{clientId}')

    @return_response
    def get_clients_count(self):
        return self.api_get_request(f'{self.TIMETAP_API_CLIENTS}/count')



from .utils import return_response, api_get_request


class ServicesMixin():
    TIMETAP_API_SERVICES = '/services'

    api_get_request = api_get_request

    @return_response
    def get_services(self):
        return self.api_get_request(f'{self.TIMETAP_API_SERVICES}')



from .utils import return_response, api_get_request


class LocationsMixin():
    TIMETAP_API_LOCATIONS = '/locations'

    api_get_request = api_get_request

    @return_response
    def get_locations(self):
        return self.api_get_request(f'{self.TIMETAP_API_LOCATIONS}')



from timetappy.appointments import AppointmentsMixin
from timetappy.locations import LocationsMixin
from timetappy.staff import StaffMixin
from timetappy.calendar import CalendarMixin
from timetappy.clients import ClientsMixin
from timetappy.services import ServicesMixin


class Client(AppointmentsMixin, LocationsMixin, StaffMixin, CalendarMixin, ClientsMixin, ServicesMixin):
    TIMETAP_BASE_URL_LIVE = "https://api.timetap.com/live"
    TIMETAP_BASE_URL_TEST = "https://api.timetap.com/test"

    def __init__(self, APIKey: str, PrivateKey: str, Testing=False):
        self.key = APIKey
        self.secret = PrivateKey
        self.testing = Testing
        self.base_url = self.TIMETAP_BASE_URL_LIVE if not self.testing else self.TIMETAP_BASE_URL_TEST

    def __repr__(self):
        return f'{type(self).__name__}(APIKey={self.key!r}, PrivateKey={self.secret!r}, Testing={self.testing!r})'



from .utils import return_response, api_get_request


class CalendarMixin():
    TIMETAP_API_CALENDAR_RANGE = '/calendar/range/'
    TIMETAP_API_CALENDAR_EVENT = '/calendar/event/'

    api_get_request = api_get_request

    @return_response
    def get_calendar_events_range(self, staffIdList: list):
        if not staffIdList:
            raise ValueError('staffIdList needs to be set to a list of `staffId`s')
        staffIdList = ','.join(map(str, staffIdList))
        return self.api_get_request(f'{self.TIMETAP_API_CALENDAR_RANGE}{staffIdList}')

    @return_response
    def get_calendar_event(self, eventType: str, eventId: int):
        if not eventType:
            raise ValueError('eventType needs to be a set to either "APPOINTMENT", "CLASS", "TIME_OFF", or "VACATION"')
        if not eventId:
            raise ValueError('eventId needs to be set to the numeric ID for an event')
        return self.api_get_request(f'{self.TIMETAP_API_CALENDAR_EVENT}{eventType}/{eventId}')
